#!/usr/bin/perl -w
#
# Use generated "site name".xml file to build a mozilla-style sidebar tab.
# If enabled, scale topic images to fit and cache locally to ease server load.
#
# Version 0.3.1 (10/24/2002)

use strict;
use File::Spec;
use Slash;
use Time::Local;
use XML::Parser;

# Image::Magick is required if you're going to resize images
use Image::Magick;

use vars qw( %task $me );

my (@lines,$GL_INSTORY,$GL_GOTSTORY,$GL_STD,$imgsrcdir,$imgdstdir,%stvars);

# Allow resizing? [0|1]
my $GL_RSZ = 1;

$task{$me}{timespec} = '15,45 * * * *';
$task{$me}{on_startup} = 1;
$task{$me}{code} = sub {
    my($virtual_user, $constants, $slashdb, $user, $info) = @_;
    my ($basedir, $sitename, $xmlfile, $output, $html, $fh, $p);
    
    $basedir = $constants->{basedir};

    $p = new XML::Parser(Handlers => {Start => \&_handle_start,
					 End   => \&_handle_end,
					 Char  => \&_handle_char
					 }
			    );
    $sitename = $constants->{sitename};
    $sitename =~ tr/[A-Z]/[a-z]/;  # could use lc instead, but who cares?
    $sitename =~ s/\W+//g;
    $xmlfile = catfile($basedir,$sitename.".xml");
    $output = catfile($basedir,"mozbar.html");
    $imgsrcdir = catfile($basedir,"images/topics/");
    $imgdstdir = catfile($basedir,"images/mozbar/");

    $p->parsefile($xmlfile);
    if ($GL_GOTSTORY){
	$html = &_generate_html($constants);
	$fh = gensym();
	open $fh, ">$output" or die "Cannot open $output for writing: $!\n";
	print $fh $html;
	close $fh;
    }
    undef @lines;
    undef $GL_GOTSTORY;    
};

sub _generate_html {
    my ($constants) = @_;
    my $sitename = $constants->{sitename};
    my ($topiclink,$imgurl,$divurl,$spcurl,$imgbase,$topicurl,$rooturl);
    my ($refresh_url,$story_lines,$i,@cur_lines,$now,$gmtime);
    $now = time;
    $gmtime = scalar(gmtime);
    $rooturl = $constants->{rootdir};
    $refresh_url = $constants->{rootdir}."/mozbar.html";
    $imgbase = $constants->{rootdir}."/images/mozbar/";
    $topicurl = $constants->{rootdir}."/search.pl";
    $divurl = $imgbase."div.png";
    $spcurl = $imgbase."spacer.png";
    for ($i = 0; $i < scalar(@lines); $i+=7){
	@cur_lines = @lines[$i..($i + 6)];
	$cur_lines[5] = &_check_freshness($cur_lines[5],$now).join(":",(split(/\:/,(split(/\ /,$cur_lines[5]))[1]))[0,1]);
	$imgurl = $imgbase.$cur_lines[1];
	$topiclink = $topicurl."?topic=".$cur_lines[0];
	if ($i) {  
            $story_lines .= <<EOT;
<tr bgcolor="#cccccc">
  <td colspan="3"><img src="$spcurl" width="1" height="1" border="0" alt="" /></td>
</tr>
EOT
        }
	$story_lines .= <<EOT;
<tr valign="middle">
  <td>
    <a href="$topiclink" target="_content"><img src="$imgurl" 
     alt="$cur_lines[2]" border="0" /></a>
  </td>
  <td>
    <a href="$cur_lines[3]" target="_content">$cur_lines[4]</a>
  </td>
  <td align="right">
    <small>$cur_lines[5]<br />
    <img src="$divurl" width="24" height="3" alt="-" /><br />
    $cur_lines[6]</small>
  </td>
</tr>
EOT
    }
    my $html = <<EOT;
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "DTD/xhtml1-transitional.dtd">
<html>
<head>
<title>$sitename Headlines</title>
<style type="text/css">
<!--
body {  
  align: top;
  padding: 3;
  border: 0;
  margin: 0;
  background-color: #c1cbd5;
     }
td {
  font-family: tahoma, arial, sans-serif;
  font-size: 11px;
   }
.inner {
  display: block; 
  overflow: auto;
  background-color: #ffffff;
  border: 1px solid #9999cc;
       }
.title {
  color: #000066;
}
a {
    text-decoration: none;
  }
a:hover {
    text-decoration: underline;
        }
small {
    font-size: 9px; color: #999999;
      }
//-->
</style>
<meta http-equiv="Refresh" content="1800; url=$refresh_url" />
<meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1" />
<meta http-equiv="Last-Modified" content="$gmtime" />
</head>
<body>
<table border="0" cellpadding="0" cellspacing="2" width="100%">
<tr class="title">
<td>&nbsp;<a href="$rooturl" class="title" target="_content">
<b>$sitename Headlines</b></a>
</td></tr>

<tr><td>
<div class="inner">
<table border="0" cellpadding="0" cellspacing="4" width="100%">

$story_lines
</table>
</div>
</td></tr>

</table>
</body>
</html>
EOT
    return $html;
}

sub _check_freshness{
    my ($datestr,$now) = @_;
    my ($date,$time) = split(/\ /,$datestr);
    my @dtary = (split(/\-/,$date))[2,1,0];
    if ($dtary[1] > 1){
	$dtary[1]--;
    }
    else{
	$dtary[1] = 11;
	$dtary[2]--;
    }
    if (($now - (timelocal((split(/\:/,$time))[2,1,0],@dtary))) > 86400){
	return "-";
    }
}

sub _handle_start{
    my ($p, $data) = @_;
    if ($data eq "story"){
	$GL_INSTORY = 1;
	$GL_GOTSTORY = 1;
    }
    else{
	$GL_STD = $data;
    }
}

sub _handle_end{
    my ($p, $data) = @_;
    my ($imgsrc, $imgdst, $img);
    if ($data eq "story"){
	push @lines, $stvars{topic}, $stvars{image}, $stvars{section}, 
	$stvars{url}, $stvars{title}, $stvars{time}, $stvars{comments};

	# Resize image, if needed
	if ($GL_RSZ){
	    $imgsrc = catfile($imgsrcdir,$stvars{image});
	    $imgdst = catfile($imgdstdir,$stvars{image});
	    $img = Image::Magick->new;
	    $img->read($imgsrc);
	    $img->Set(type=>'TrueColorMatte');
	    $img->Resize(geometry=>'24x24', blur=>0);
	    $img->Write($imgdst);
	}
	undef %stvars;
    }
    else{
	$GL_STD = "";
    }
}

sub _handle_char{
    my ($p, $data) = @_;
    if ($GL_STD){
	$stvars{$GL_STD} = $data;
    }
}

1;

